/**
 * 判断富文本是否是一个空，用来判断富文本是否为空
 *
 * @param {cont} 富文本对象
 */
export function isAnEmptyRichText(cont) {
  if (!cont) return true;

  if (!cont.sections || !cont.sections.length) return true;

  let blocks = [];
  cont.sections.forEach((section) => {
    blocks = [...blocks, ...section.blocks];
  });

  if (!blocks.length) return true;

  if (blocks.some((item) => item.type !== "text")) {
    return false;
  } else {
    return !blocks
      .map((item) => item.value)
      .join("")
      .trim();
  }
}

/**
 * 判断富文本是否包含音频
 * @param  cont 富文本对象
 * @returns Boolean
 */
export function checkRichTextHasAudio(cont) {
  if (isAnEmptyRichText(cont)) return false;

  return cont.sections.some((section) =>
    section.blocks.some((block) => block.type === "audio")
  );
}

/**
 * 数字转大写字母
 * @param {number} val 数字
 * @returns 字母
 */
export function numberToUpperCase(val) {
  if (val < 1 || val > 26) return;

  return String.fromCharCode(64 + val);
}

/**
 * 获取富文本数据中答题点个数
 * @param {richText} richText
 * @returns
 */
export function getRichTextAnswerPointCount(richText) {
  if (isAnEmptyRichText(richText)) return 0;

  let count = 0;
  richText.sections.forEach((section) => {
    section.blocks.forEach((block) => {
      if (block.type === "cloze") count++;
    });
  });

  return count;
}
